import re

N179 = "N179 Do not return 'YES'/'NO', use return True/False"

yes_no_regex = re.compile(r'''\s*return\s*['"](?:yes|no)['"]\s*''', flags=re.IGNORECASE)


def returnyesno(physical_line):
    m = yes_no_regex.fullmatch(physical_line)
    if m:
        ps = physical_line.find('return')
        return (ps + 7, N179)


returnyesno.name = 'return_yes_no_checker'
returnyesno.version = '0.1'



from flake8.formatting import base
from flake8_ejudge.flake8_ruerrors import code_to_ru_text, text_to_ru_text
from flake8.violation import Violation

LINE_STARTER = '>Line '

_SEPS = ['"', '(', ')', "'"]
_ERROR_FORMAT = LINE_STARTER + '{row}: {code} '


def _extract_template(code, text):
    seps = [(i, c) for i, c in enumerate(text) if c in _SEPS]
    # Если у нас один сепаратор внутри других сепараторов, то он — не сепаратор.
    for i in range(len(seps) - 2, 0, -1):
        i1, c1 = seps[i - 1]
        i2, c2 = seps[i]
        i3, c3 = seps[i + 1]
        if i1 + 1 == i2 == i3 - 1 and c1 == c3 != c2:
            seps.pop(i)
    replacers = []
    if seps:
        if len(seps) % 2 != 0:
            # print('Неуравновешено:', err_text)
            pass
        else:
            replacers = [text[seps[i][0]:seps[i + 1][0] + 1] for i in range(0, len(seps), 2)]
            for i in range(len(seps) - 2, -1, -2):
                ff = seps[i][0]
                tt = seps[i + 1][0]
                text = text[:ff] + '{' + str(i // 2) + '}' + text[tt + 1:]
    if code == 'F811':  # redefinition of unused $0 from line 273
        rsp = text.rfind(' ')
        replacers.append(text[rsp + 1:])
        text = text[:rsp + 1] + '{1}'
    elif code in ('E301', 'E302', 'E305', 'E306'):  # expected 2 blank lines after class or function definition, found
        rsp = text.rfind(' ')
        replacers.append(text[rsp + 1:])
        text = text[:rsp + 1] + '{0}'
    elif code == 'F405':  # $0 may be undefined, or defined from star imports:
        rsp = text.rfind('imports:')
        replacers.append(text[rsp + 9:])
        text = text[:rsp + 9] + '{1}'
    elif code == 'E712':  # comparison to False should be {0} or {1}
        if 'False' in text:
            text = text.replace('False', '{2}')
            replacers.append('False')
        elif 'True' in text:
            text = text.replace('True', '{2}')
            replacers.append('True')

    return text, replacers


_extract_template('E211', "whitespace before '('")


class Flake8EjudgeFormatter(base.BaseFormatter):
    """Flake8's formatter."""

    def format(self, error):
        code = error.code
        text = error.text
        path = error.filename
        row = error.line_number
        col = error.column_number
        text_r, replacers = _extract_template(code, text)
        if code in code_to_ru_text:
            try:
                rutext = code_to_ru_text[code].format(*replacers)
            except:
                rutext = code_to_ru_text[code]
        elif text_r in text_to_ru_text:
            try:
                rutext = text_to_ru_text[text_r].format(*replacers)
            except:
                rutext = text_to_ru_text[text_r]
        else:
            rutext = ''
        st = _ERROR_FORMAT.format(**locals())
        res = st + text
        if rutext:
            res += '\n' + ' ' * len(st) + rutext
        return res

    def show_source(self, error: Violation):
        """Show the physical line generating the error.

        This also adds an indicator for the particular part of the line that
        is reported as generating the problem.

        :param error:
            This will be an instance of
            :class:`~flake8.violation.Violation`.
        :returns:
            The formatted error string if the user wants to show the source.
            If the user does not want to show the source, this will return
            ``None``.
        """
        if not self.options.show_source or error.physical_line is None:
            return ""

        # Because column numbers are 1-indexed, we need to remove one to get
        # the proper number of space characters.
        indent = "".join(
            c if c.isspace() else " "
            for c in error.physical_line[: error.column_number - 1]
        )
        physical_line = error.physical_line.rstrip()
        return f"{physical_line}\n{indent}^"



# см. https://lintlyci.github.io/Flake8Rules/
code_to_ru_text = {
    'A001': 'Имя {0} уже используется в python. Назовите как-нибудь по-другому или добавьте "_" в конец',
    'A002': 'Имя {0} уже используется в python. Назовите как-нибудь по-другому или добавьте "_" в конец',
    'A003': 'Имя {0} уже используется в python. Назовите как-нибудь по-другому или добавьте "_" в конец',
    'C901': 'Цикломатическая сложность функции {0} (или всей программы) слишком велика: {1}. Разбейте её на подфункции, сделайте логику более линейной.',
    'E101': 'Отступ содержит и пробелы, и табуляции. Замените всё на пробелы.',
    'E111': 'Величина отступа должна делиться на 4',
    'E112': 'Здесь должен быть сдвинутый блок',
    'E113': 'Здесь не может быть такого отступа',
    'E114': 'Величина отступа должна делиться на 4, а не делится: {0}',
    'E115': 'Здесь должен быть сдвинутый блок {0}',
    'E116': 'Здесь не может быть такого отступа {0}',
    'E117': 'Слишком большой отступ. Должен быть самый маленький из делящихся на 4. Например, 4, а не 5 и не 8.',
    'E121': 'Строчка продолжает предыдущую со слишком маленьким отступом. Сделайте хотя бы 4.',
    'E122': 'Строчка продолжает предыдущую без отступа. Сделайте хотя бы 4.',
    'E123': "Закрывающая скобка в таком контексте должна стоять на том же уровне, что и начало строчки, в которой эта скобка открыта",
    'E124': 'Закрывающая скобка в таком контексте должна стоять на том же уровне, что и её открывающая',
    'E126': 'continuation line over-indented for hanging indent',
    'E127': 'continuation line over-indented for visual indent',
    'E128': 'continuation line under-indented for visual indent',
    'E129': 'visually indented line with same indent as next logical line',
    'E131': 'continuation line unaligned for hanging indent',
    'E201': 'Удалите пробел после {0}',
    'E202': 'Удалите пробел перед {0}',
    'E203': 'Удалите пробел перед {0}',
    'E211': 'Удалите пробел перед {0}',
    'E221': 'Тут больше 1 пробела, оставьте ровно 1',
    'E222': 'Тут больше 1 пробела, оставьте ровно 1',
    'E225': 'Поставьте пробелы вокруг оператора',
    'E226': 'Поставьте пробелы вокруг оператора',
    'E227': 'Поставьте пробелы вокруг оператора',
    'E228': 'Поставьте пробелы вокруг оператора',
    'E231': 'Добавьте пробел после {0}',
    'E241': 'Тут после {0} больше 1 пробела, оставьте ровно 1',
    'E251': 'В параметрах знак «=» пробелами не отделяется, удалите пробелы',
    'E261': 'Перед строчным комментарием поставьте 2 пробела',
    'E262': 'Строчный комментарий должен начинаться с {0}',
    'E265': 'Блочный комментарий должен начинаться с {0}',
    'E266': 'Слишком много символов "#" для комментария. Оставьте ровно 1',
    'E271': 'Оставьте ровно 1 пробел после ключевого слова',
    'E272': 'Оставьте ровно 1 пробел перед ключевым словов',
    'E275': 'Поставьте ровно 1 пробел после ключевого слова',
    'E301': 'Между методами класса оставляйте ровно 1 пустую строку (а не {0})',
    'E302': 'Между функциями и классами делайте ровно 2 пустых строчки (а не {0})',
    'E303': 'Слишком много пустых строк: {0}. Между функциями ставьте 2, между методами класса — 1.',
    'E304': 'Удалите пустую строку между декоратором и функцией',
    'E305': 'После функций и классов вставляйте ровно 2 пустых строки (а не {0})',
    'E306': 'Вложенные функции отделяйте ровно 1 пустой строкой (а не {0})',
    'E401': 'Разнесите импорты на несколько строчек',
    'E402': 'Импорт модулей перенесите в самое начало',
    'E501': 'Эта строчка слишком длинная: {0}. Разбейте её на несколько, используйте промежуточные переменные.',
    'E502': 'Удалите "\\", в данном контексте он не нужен',
    'E701': 'Разные команды должны быть на разных строчках. Так код лучше читается.',
    'E702': 'Разные команды должны быть на разных строчках. Так код лучше читается.',
    'E703': 'Удалите ";", точка с запятой не нужны в питоне',
    'E704': 'Разные команды должны быть на разных строчках. Так код лучше читается.',
    'E711': 'Сравнивать с None надо так: "if var is None"',
    'E712': 'Сравнивать с True/False надо так: "if var:" или "if not var:". Или так "if var is True:", если важна проверка типа',
    'E713': 'Лучше писать не "not var in list", а "var not in list", так легче читать',
    'E714': 'Лучше писать не "not var is None", а "var is not None", так легче читать',
    'E721': 'Лучше не сравнивать типы "type(var)==...", и спользовать isinstance: "isinstance(var, ...)"',
    'E722': "Лучше в except указывать список отлавливаемых ошибок (\"except SomeError\" или \"except (Err1, Err2)\"",
    'E731': 'Используйте def для определения функции, не присваивайте lambda функции переменным',
    'E741': 'Не используйте «I», «O» и «l» в качестве имён переменных, их легко спутать с 1 или 0',
    'E743': 'Не используйте «I», «O» и «l» в качестве имён функций, их легко спутать с 1 или 0',
    'F401': 'Объект {0} импортирован, но не использован. Удалите его',
    'F403': 'Не используйте "from module import *". Укажите явно список импортированных объектов ("from module import fun, obj")',
    'F405': 'Возможно, переменная {0} не инициализирована (или импортирована неявно "from module import *")',
    'F406': '‘from module import *’ нельзя делать внутри функии. Да и вообще лучше писать "from module import fun, obj" в начале модуля',
    'F601': 'В словаре ключ {0} перезаписывается разными значениями',
    'F701': 'break нельзя использовать вне цикла',
    'F706': 'return нельзя использовать вне функции',
    'F811': 'Здесь переопределение {0} из строчки {1}. Удалите одно из них.',
    'F821': 'Переменная {0} не определена там, где используется. Определите её',
    'F823': 'Локальная переменная {0}{1} используется до объявления. Определите её',
    'F841': 'Локальная переменная {0} определена, но нигде не используется. Удалите её',
    'W191': 'Отступ содержит табуляции. Замените на пробелы',
    'W291': 'Удалите пробелы из конца строки',
    'W292': 'Добавьте ровно 1 пустую строчку в конец файла',
    'W293': 'Из пустой строки удалите все пробелы и табуляции',
    'W391': 'Удалите лишние пустые строчки из конца файла, оставьте ровно 1',
    'W503': 'Сделайте перенос строки после бинарного оператора, а не до',
    'N179': 'Вместо return "YES"/"NO" делайте return True/False'
}

text_to_ru_text = {
    "IndentationError: unindent does not match any outer indentation level": 'Отступ не соответствует ни одному использованному, исправьте',
    "TokenError: EOF in multi-line statement": 'Многострочное выражение прерывается в конце файла, что-то не закрыто или не дописано',
    "SyntaxError: can't assign to comparison": 'Нельзя присваивать сравнению',
    "SyntaxError: can't assign to function call": 'Нельзя присваивать вызову функции',
    "SyntaxError: can't assign to operator": 'Нельзя присваивать оператору',
    "SyntaxError: keyword can't be an expression": 'Нельзя использовать ключевое слово',
    "IndentationError: expected an indented block": 'Здесь должен быть отступ',
    "IndentationError: unexpected indent": 'Некорректный отступ',
    "SyntaxError: EOL while scanning string literal": 'Пропущена кавычка, завершающая строку',
    "SyntaxError: invalid character in identifier": 'Некорректный символ в имени',
    "SyntaxError: invalid syntax": 'Неверный синтаксис',
    "SyntaxError: invalid token": 'Недопустимый токен',
    "SyntaxError: positional argument follows keyword argument": 'После параметра, переданного по имени, могут идти только параметры, переданные по имени',
    "SyntaxError: unexpected character after line continuation character": 'Недопустимый символ после символа продолжения строки',
    "SyntaxError: unexpected EOF while parsing": 'Что-то не закрыто или не дописано',
}



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import sys
import os
import re
from flake8_ejudge.flake8_ejudgeformatter import LINE_STARTER
from flake8.main import application
import io
import base64

FLAKE = 'flake8'
MAX_LEN = 160
IGNORE = 'W293,W292,W291,W391,F405,E722,E743,E101,F403,E721,E226'
TIMEOUT = 5
MAX_COMPLEXITY = 9
MAX_ERRORS_TO_SHOW = 10


def parse_evs(evs: dict):
    # Дополняем стандартные параметры значениями из env
    flake_parms = [
        '--show-source',
        '--jobs=1',
        '--isolated',
        '--format=flake8ejudgeformatter',
    ]
    if 'flake8_ignore' in evs:
        flake_parms.append('--ignore=' + evs['flake8_ignore'].replace(' ', '').upper())
    else:
        flake_parms.append('--ignore=' + IGNORE)

    if 'max_line_length' in evs:
        flake_parms.append('--max-line-length=' + evs['max_line_length'])
    else:
        flake_parms.append('--max-line-length=' + str(MAX_LEN))

    if 'max_complexity' in evs and evs['max_complexity'].isdecimal() and int(evs['max_complexity']) < 99:
        flake_parms.append('--max-complexity=' + evs['max_complexity'])
    elif 'max_complexity' in evs and evs['max_complexity'].isdecimal() and int(evs['max_complexity']) >= 99:
        pass
    else:
        flake_parms.append('--max-complexity=' + str(MAX_COMPLEXITY))

    max_errors = int(evs.get('max_errors_to_show', MAX_ERRORS_TO_SHOW))
    return flake_parms, max_errors


def run_flake8(src_name: str, evs: dict):
    errors_found = False
    flake_parms, max_errors = parse_evs(evs)

    # Делаем так, чтобы flake8 налогировал нам в переменную
    old_stdout = sys.stdout
    log_capture_string = io.BytesIO()
    sys.stdout = log_capture_string
    sys.stdout.buffer = log_capture_string
    # Запускаем flake
    app = application.Application()
    # app.initialize(flake_parms)
    # app.run_checks([src_name])
    app.run([src_name, *flake_parms])
    # app.report_errors()
    # Вытягиваем данные
    sys.stdout = old_stdout
    stdout_data = log_capture_string.getvalue().decode('utf-8')
    log_capture_string.close()
    if stdout_data:
        stdout_data = '\n' + stdout_data
        tot_errors = stdout_data.count('\n' + LINE_STARTER)
        if tot_errors > max_errors:
            ps = -1
            for __ in range(max_errors):
                ps = stdout_data.find('\n' + LINE_STARTER, ps + 1)
            stdout_data = stdout_data[:ps]
            rem = tot_errors - max_errors
            tail = 'ий' if 5 <= rem % 100 <= 20 else 'ия' if 1 < rem % 10 < 5 else 'ие'
            add = '\n' + '=' * 50 + \
                  '\nТам ещё {} замечан{} к стилю.\n'.format(rem, tail) + \
                  'Используйте автоформатирование кода: Ctrl+Alt+L в PyCharm или сервис https://black.now.sh'
            stdout_data += add
        errors_found = True
    else:
        stdout_data = ' '
    return errors_found, stdout_data[1:]


def run_regex_checks(src_name: str, evs: dict):
    errors_found = False
    stdout_data = ''

    # Читаем исходники
    try:
        with open(src_name, 'r', encoding="utf-8", errors='ignore') as f:
            source = f.read()
    except:
        stdout_data = 'Can not parse source with encoding utf-8'
        errors_found = True
        return errors_found, stdout_data

    # Теперь вычитываем дополнительные настройки
    for i in range(1, 30):
        pattern, in_or_not, msg = 'chk_pattern_' + str(i), 'chk_in_or_not_' + str(i), 'chk_err_msg_' + str(i)
        check = [evs.get(x, '') for x in (pattern, in_or_not, msg)]
        if not all(check):
            continue
        pattern, in_or_not, msg = check
        if pattern.startswith('b32_'):
            pattern = base64.b32decode(pattern[4:].upper().encode()).decode('utf-8')

        check_found = bool(re.search(pattern, source))
        if (check_found and in_or_not != 'in') or (not check_found and in_or_not != 'not'):
            stdout_data += msg + '\n'
            errors_found = True
    return errors_found, stdout_data


def style_check(src_name: str, f_obj):
    # Если имя файла не заканчивается на .py, то игнорируем
    if not src_name.lower().endswith('.py'):
        return 0
    # Проверяем, что это действительно файл и у нас есть к нему доступ
    if not os.path.isfile(src_name):
        return 0

    # Заменяем все "-" на "_" и убираем ведущие "--", если они были
    evs = {key.replace("-", '_').lstrip('_').lower(): val for key, val in os.environ.items()}

    # Запускаем flake8
    errors_found_flake = False
    if 'regexonly' not in evs:
        errors_found_flake, stdout_data_flake = run_flake8(src_name, evs)
        f_obj.write(stdout_data_flake)
    # Запускаем проверки по регуляркам
    errors_found_regex, stdout_data_regex = run_regex_checks(src_name, evs)
    f_obj.write(stdout_data_regex)

    # Выходим с ошибкой или без в зависимости от.
    if errors_found_flake or errors_found_regex:
        return 1


def main():
    # ejudge вызывает валидатор с единственным параметром — именем файла, которому необходима проверка.
    if len(sys.argv) < 2:
        sys.stderr.write('Usage: flake8ejudge filename\n')
        sys.exit(1)
    src_name = sys.argv[1]
    exit_code = style_check(src_name, f_obj=sys.stderr)
    sys.exit(exit_code)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pip install twine

# build and upload
# pip install wheel

# python setup.py sdist bdist_wheel
# twine check dist/*
# twine upload dist/*


import io
import re
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = 'flake8_ejudge'
DESCRIPTION = 'Проверка стиля и требований к используемому синтаксису в python коде для ejudge'
URL = 'https://github.com/ShashkovS/flake8_ejudge'
EMAIL = 'sh57@yandex.ru'
AUTHOR = 'Sergey Shashkov'

here = os.path.abspath(os.path.dirname(__file__))
# Current version
with io.open(os.path.join(here, 'src', NAME, '__init__.py'), encoding='utf-8') as f:
    VERSION = re.search(r'\d+\.\d+\.\d+', f.read()).group()

# What packages are required for this module to be executed?
with io.open(os.path.join(here, 'requirements.txt'), encoding='utf-8') as f:
    REQUIRED = [row.strip() for row in f.readlines() if row.strip() and not row.strip().startswith('#')]

# Import the README and use it as the long-description.
# Note: this will only work if 'README.rst' is present in your MANIFEST.in file!
with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    LONG_DESCRIPTION = '\n' + f.read()


class PublishCommand(Command):
    """Support setup.py publish."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/x-rst',
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    packages=find_packages(where='src'),
    package_dir={"": "src"},
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],

    entry_points={
        'console_scripts': ['flake8ejudge=flake8_ejudge.flake8_ejudge_runner:main'],
        'flake8.report': ['flake8ejudgeformatter = flake8_ejudge.flake8_ejudgeformatter:Flake8EjudgeFormatter'],
        'flake8.extension': ['N179 = flake8_ejudge.flake8_returnyesno:returnyesno'],
    },
    install_requires=REQUIRED,
    include_package_data=True,
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Framework :: Flake8',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: Russian',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Quality Assurance',
    ],
    # $ setup.py publish support.
    cmdclass={
        'publish': PublishCommand,
    },
)
